/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include "nrf.h"
#include "nrf_delay.h"
#include "twi_master_int.h"

#define TWI_ADDRESS     0x3E

static uint8_t twi_data_tx[2];
static uint8_t twi_data_rx[2];
    
/**
 * main function
 * \return 0. int return type required by ANSI/ISO standard. 
 */
int main(void)
{
    bool return_val = true;
    
    // --- Blocking example ---
    // (TWI functions will not return until the entire TWI operation is complete)
    twi_config_t my_twi_config = {.pinselect_scl = 24,
                                  .pinselect_sda = 25,
                                  .frequency     = TWI_FREQ_100KHZ,
                                  .blocking_mode = TWI_BLOCKING_ENABLED};
    
    return_val &= twi_master_init(&my_twi_config);
    
    // Write operation
    twi_data_tx[0] = 0x00;
    twi_data_tx[1] = 0x01;
    return_val &= twi_master_write(TWI_ADDRESS, twi_data_tx, 2);
                                  
    // Read operation
    return_val &= twi_master_read(TWI_ADDRESS, twi_data_rx, 2);
 
    // Combined write/read operation (repeated start)
    return_val &= twi_master_write_read(TWI_ADDRESS, twi_data_tx, 2, twi_data_rx, 2); 

    if(return_val)
    {
        // All operations successful
    }
    
    // --- Non blocking example ---
    //(TWI functions will return after initializing the TWI lib, and the MCU can do other tasks in the mean time
    my_twi_config.blocking_mode = TWI_BLOCKING_DISABLED;
    
    return_val = true;
    return_val &= twi_master_init(&my_twi_config);
                                  
    // Write operation
    twi_data_tx[0] = 0x00;
    twi_data_tx[1] = 0x01;
    twi_master_write(TWI_ADDRESS, twi_data_tx, 2);
    
    // The MCU can do other tasks while the TWI interface is running in the background                                     
                                  
    // To check the result of the last TWI operation, and wait for it to complete, call twi_wait_for_completion()
    return_val &= twi_wait_for_completion();
                                  
    // Read operation
    twi_master_read(TWI_ADDRESS, twi_data_rx, 2);
    
    // The MCU can do other tasks while the TWI interface is running in the background    
                                  
    return_val &= twi_wait_for_completion();
 
    // Combined write/read operation (repeated start)
    twi_master_write_read(TWI_ADDRESS, twi_data_tx, 2, twi_data_rx, 2);    

    // The MCU can do other tasks while the TWI interface is running in the background   
    
    return_val &= twi_wait_for_completion();
    
    if(return_val)
    {
        // All non-blocking operations successful
    }
    
    while(1);
}

/**
 *@}
 **/
